﻿using Microsoft.AspNetCore.Mvc.Filters;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace P2Pdemo.Models
{
    public class PermissionFilter : ActionFilterAttribute
    {
        public override void OnActionExecuting(ActionExecutingContext context)
        {
            try
            {
                var obj = context.ActionArguments.Values.ToList()[0];
                string param = TyziTools.ApiSign.ToString(obj);
                if (string.IsNullOrEmpty(param) || param.Contains("ValueKind"))
                    param = TyziTools.ApiSign.ToString(System.Text.Json.JsonSerializer.Deserialize<Dictionary<string, string>>(obj.ToString()));

                if (!string.IsNullOrEmpty(param))
                {
                    Dictionary<string, object> pairs = TyziTools.ApiSign.ToObject<Dictionary<string, object>>(param.ToString());
                    string key = pairs.ContainsKey("key") ? pairs["key"].ToString() : "";
                    if (string.IsNullOrEmpty(key))
                    {
                        key = pairs.ContainsKey("deviceno") ? pairs["deviceno"].ToString() : "";
                    }
                    if ((pairs.ContainsKey("deviceno") || pairs.ContainsKey("key")) && pairs.ContainsKey("sign") && pairs.ContainsKey("timestamp") && pairs.ContainsKey("data"))
                    {
                        string signKey = pairs.ContainsKey("deviceno") ? "deviceno" : "key";
                        pairs["data"] = AESHelper.AesDecrypt(System.Web.HttpUtility.UrlDecode(pairs["data"].ToString()));
                        bool ok = ResultHelper.SignChecker(pairs, signKey);
                        if (!ok)
                        {
                            context.Result = ResultHelper.AESSendResult(key, ((int)HttpStatusCode.Forbidden).ToString(), "访问限制");
                            return;
                        }

                        if (TyziTools.ApiSign.CheckTimeRange(pairs["timestamp"].ToString(), 1))
                        {
                            context.Result = ResultHelper.AESSendResult(key, "7", "时间戳错误", DateTime.Now.ToString("yyyyMMddHHmmssff"));
                            return;
                        }
                    }
                    else
                    {
                        context.Result = ResultHelper.AESSendResult(key, ((int)HttpStatusCode.NotAcceptable).ToString(), "请求参数错误");
                    }
                }
                else
                {
                    context.Result = ResultHelper.AESSendResult("", ((int)HttpStatusCode.NotFound).ToString(), "未发现请求路径");
                }
                //}
            }
            catch (Exception ex)
            {
                context.Result = ResultHelper.AESSendResult("", ((int)HttpStatusCode.InternalServerError).ToString(), "信息异常：" + ex.Message);
            }
        }
    }
}
