﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using P2Pdemo.Models;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace P2Pdemo.Controllers
{    
    public class IndexController : Controller
    {
        public class ResultBase
        {
            public string Code { get; set; }
            public bool Success { get; set; }
            public string Message { get; set; }
            public object Data { get; set; }
        }

        public class DeviceResultBase
        {
            public int resultcode { get; set; } = -1;
            public string msg { get; set; } = "";
            public string actionName { get; set; } = "";
            public object data { get; set; } = "";
        }

        public ResultBase RB = new ResultBase();

        public IActionResult Index()
        {
            return View();
        }

        /// <summary>
        /// 接收对讲
        /// </summary>
        /// <param name="jsonModel"></param>
        public void ReceiveTalking(string terminalNo,string account,string pwd)
        {
            try
            {

                Dictionary<string, object> data = new Dictionary<string, object>();
                data.Add("actionName", "startIntercom");//把对讲设备对讲请求发送给前端
                data.Add("terminalNo", terminalNo);//机器人编号                
                data.Add("talkType", 2);
                    data.Add("account", account);//对讲方式
                    data.Add("pwd", pwd);//对讲方式
                    data.Add("ppsvraddr", AppSettingConfig.PP_SvrAddr);                
                DeviceResultBase resultBase = HttpTimeWebSocketSend(AppSettingConfig.Talk_Server, "pullDataStr", terminalNo, data);
                if (resultBase.resultcode==-1)
                {
                    RB.Success = false;
                }
                else
                {
                    RB.Success = true;
                    RB.Data = data;//房间号                              
                }              
            }
            catch (Exception ex)
            {
                RB.Success = false;
                RB.Message = "系统错误，" + ex.Message;
            }
            Response.WriteAsync(JsonConvert.SerializeObject(RB));
        }

        /// <summary>
        /// 线上关闭对讲
        /// </summary>
        /// <param name="jsonModel"></param>
        public void OnlineCloseTalking(string terminalNo)
        {
            DeviceResultBase resultBase = new DeviceResultBase();
            try
            {
                Dictionary<string, object> data = new Dictionary<string, object>();
                data.Add("actionName", "stopIntercom");//把对讲设备对讲请求发送给前端
                data.Add("terminalNo", terminalNo);//机器人编号               
                data.Add("talkType", 2);//车场编号                    
                resultBase = HttpTimeWebSocketSend(AppSettingConfig.Talk_Server, "pullDataStr", terminalNo, data);
                if (resultBase.resultcode == -1)
                {
                    RB.Success = false;                  
                }
                else
                {
                    RB.Success = true;
                    RB.Data = data;//房间号
                }
            }
            catch (Exception ex)
            {
                RB.Success = false;
                RB.Message = "系统错误，" + ex.Message;               
            }
            Response.WriteAsync(JsonConvert.SerializeObject(RB));
        }

        public static DeviceResultBase HttpTimeWebSocketSend(string url, string actionName, string deviceNo, Dictionary<string, object> data)
        {
            string result = "";

            DeviceResultBase resultBase = new DeviceResultBase();

            try
            {
                url = url.TrimEnd('/') + "/" + actionName + "?terminalId=" + deviceNo + "&pullDataStr=" +
                      HttpUtility.UrlEncode(JsonConvert.SerializeObject(data).Replace("\r\n", ""), Encoding.UTF8);

                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
                request.Timeout = 6000;
                request.Method = "GET";
                request.ContentType = "application/json;charset=UTF-8";
                request.ServicePoint.ConnectionLimit = int.MaxValue;
                request.KeepAlive = false;
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream myResponseStream = response.GetResponseStream();
                StreamReader myStreamReader = new StreamReader(myResponseStream, Encoding.UTF8);
                result = myStreamReader.ReadToEnd();
                JObject jObject = JObject.Parse(result);

                myStreamReader.Close();
                myResponseStream.Close();
                response.Close();

                //兼容线下中间件V1.5之前版本，加relustcode,resultcode判断
                if ((jObject["result"] != null && Convert.ToInt32(jObject["result"].ToString()) == 1)
                    || (jObject["relustcode"] != null && Convert.ToInt32(jObject["relustcode"].ToString()) == 1)
                    || (jObject["resultcode"] != null && Convert.ToInt32(jObject["resultcode"].ToString()) == 1)
                    || (jObject["relust"] != null && Convert.ToInt32(jObject["relust"].ToString()) == 1))
                {
                    resultBase.resultcode = 1;
                    resultBase.msg = jObject["msg"].ToString();
                    if (jObject["data"] != null)
                    {
                        JObject dataObj = JsonConvert.DeserializeObject<JObject>(jObject["data"].ToString());
                        if (dataObj != null)
                            resultBase.data = jObject["data"];
                        else
                            resultBase.data = null;
                    }
                    if (jObject["actionName"] != null)
                    {
                        resultBase.actionName = jObject["actionName"].ToString();
                    }
                }
                else
                {
                    resultBase.resultcode = -1;
                    resultBase.msg = jObject["msg"].ToString();
                    if (jObject["data"] != null)
                    {
                        JObject dataObj = JsonConvert.DeserializeObject<JObject>(jObject["data"].ToString());
                        if (string.IsNullOrEmpty(jObject["data"].ToString()) || dataObj == null)
                        {
                            resultBase.data = null;
                        }
                        else
                        {
                            resultBase.data = jObject["data"];
                        }
                    }

                    if (jObject["actionName"] != null)
                        resultBase.actionName = jObject["actionName"].ToString();
                }
            }
            catch (Exception ex)
            {
                resultBase.resultcode = -2;
                resultBase.msg = $"Http消息实时下发错误:{ex.ToString()},【请求URL：{url},返回Result：{result}】";
            }

            return resultBase;
        }

    }
}
