﻿using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using P2Pdemo.Models;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace P2Pdemo.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    [TypeFilter(typeof(PermissionFilter))]
    public class TalkingApiController : Controller
    {
        public class DeviceResultBase
        {
            public int resultcode { get; set; } = -1;
            public string msg { get; set; } = "";
            public string actionName { get; set; } = "";
            public object data { get; set; } = "";
        }

        #region 获取车道助手参数

        /// <summary>
        /// 获取车道助手配置信息
        /// </summary>
        /// <param name="jsonData">{"deviceno":""}</param>
        /// <returns></returns>
        [HttpPost("LaneHelperConfig")]
        public ActionResult LaneHelperConfig([FromBody] Models.ParamModel param)
        {

            string keyWordText = "";
            try
            {
                keyWordText =
                $"{param.deviceno.ToString()}";

                CommHelper.AESDataParam(ref param);

                string LaneHelper_Key = param.deviceno.ToString();
                string msg = string.Empty;

                Dictionary<string, object> dic = new Dictionary<string, object>();

                dic.Add("ppsvraddr", AppSettingConfig.PP_SvrAddr);
                dic.Add("tcpip", AppSettingConfig.Talk_TcpIP);
                dic.Add("tcpport", AppSettingConfig.Talk_TcpPort); ;

                return ResultHelper.AESSendResult(keyWordText, "1", "请求成功", dic);

            }
            catch (Exception ex)
            {
                return ResultHelper.AESSendResult(keyWordText, "0", "获取车道助手配置信息系统异常，" + ex.Message);
            }
        }
        #endregion

        #region 接收车道助手呼叫(车道助手呼叫)

        /// <summary>
        /// 接收车道助手呼叫
        /// </summary>
        /// <param name="jsonData">{"deviceno":""}</param>
        /// <returns></returns>
        [HttpPost("StartTalking")]
        public ActionResult StartTalking([FromBody] Models.ParamModel param)
        {
            DeviceResultBase resultBase = new DeviceResultBase();
            string keyWordText = "";
            try
            {
                CommHelper.AESDataParam(ref param);
                string msg = string.Empty;

                Dictionary<string, object> data = new Dictionary<string, object>();
                data.Add("actionName", "startIntercom");//把对讲设备对讲请求发送给前端
                data.Add("terminalNo", param.deviceno.ToString());//呼叫编号  type-1 呼叫助手编号 2-小程序编号                 
                data.Add("type", 1);//呼叫类型 1-通道机                                      
                data.Add("talkType", 2);//对讲方式
                DeviceResultBase resultUser = HttpTimeWebSocketSend(AppSettingConfig.Talk_Server, "Talking", "admin", data);

                if (resultUser.resultcode != -1) 
                {

                    return ResultHelper.AESSendResult(keyWordText, "1", "接收车道助手呼叫请求成功(车道助手呼叫)");
                }
                else
                {

                    return ResultHelper.AESSendResult(keyWordText, "0", "接收车道助手呼叫请求失败(车道助手呼叫)：" + msg);
                }
            }
            catch (Exception ex)
            {

                return ResultHelper.AESSendResult(keyWordText, "0", "接收车道助手呼叫请求异常(车道助手呼叫)：" + ex.Message);
            }

        }

        #endregion

        #region 关闭对讲请求(车道助手关闭对讲)

        /// <summary>
        /// 关闭对讲请求(车道助手关闭)
        /// </summary>
        /// <param name="jsonData">{"DeviceNo":""}</param>
        /// <returns></returns>
        [HttpPost("StopTalking")]
        public ActionResult StopTalking([FromBody] Models.ParamModel param)
        {
            DeviceResultBase resultBase = new DeviceResultBase();
            string keyWordText = "";
            try
            {
                keyWordText =
                    $"{param.deviceno.ToString()}";
                CommHelper.AESDataParam(ref param);
              

                string msg = string.Empty;
                Dictionary<string, object> data = new Dictionary<string, object>();
                data.Add("actionName", "stopIntercom");//action
                data.Add("terminalNo", param.deviceno.ToString());//呼叫编号  type-1 呼叫助手编号 2-小程序编号                       
                data.Add("type", 1);//呼叫类型
                data.Add("talkType", 2);//对讲方式
                DeviceResultBase resultUser =HttpTimeWebSocketSend(AppSettingConfig.Talk_Server, "Talking", "admin", data);

                if (resultUser.resultcode != -1) 
                {
                    return ResultHelper.AESSendResult(keyWordText, "1", "接收车道助手关闭对讲请求成功(车道助手关闭)");
                }
                else
                {
                    return ResultHelper.AESSendResult(keyWordText, "0", "接收车道助手关闭对讲请求失败(车道助手关闭)：" + msg);
                }

            }
            catch (Exception ex)
            {
                
                return ResultHelper.AESSendResult(keyWordText, "0", "道助手关闭对讲请求异常(车道助手关闭)：" + ex.Message);
            }
        }

        #endregion

        public static DeviceResultBase HttpTimeWebSocketSend(string url, string actionName, string deviceNo, Dictionary<string, object> data)
        {
            string result = "";

            DeviceResultBase resultBase = new DeviceResultBase();

            try
            {
                url = url.TrimEnd('/') + "/" + actionName + "?terminalId=" + deviceNo + "&pullDataStr=" +
                      HttpUtility.UrlEncode(JsonConvert.SerializeObject(data).Replace("\r\n", ""), Encoding.UTF8);

                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
                request.Timeout = 6000;
                request.Method = "GET";
                request.ContentType = "application/json;charset=UTF-8";
                request.ServicePoint.ConnectionLimit = int.MaxValue;
                request.KeepAlive = false;
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                Stream myResponseStream = response.GetResponseStream();
                StreamReader myStreamReader = new StreamReader(myResponseStream, Encoding.UTF8);
                result = myStreamReader.ReadToEnd();
                JObject jObject = JObject.Parse(result);

                myStreamReader.Close();
                myResponseStream.Close();
                response.Close();

                //兼容线下中间件V1.5之前版本，加relustcode,resultcode判断
                if ((jObject["result"] != null && Convert.ToInt32(jObject["result"].ToString()) == 1)
                    || (jObject["relustcode"] != null && Convert.ToInt32(jObject["relustcode"].ToString()) == 1)
                    || (jObject["resultcode"] != null && Convert.ToInt32(jObject["resultcode"].ToString()) == 1)
                    || (jObject["relust"] != null && Convert.ToInt32(jObject["relust"].ToString()) == 1))
                {
                    resultBase.resultcode = 1;
                    resultBase.msg = jObject["msg"].ToString();
                    if (jObject["data"] != null)
                    {
                        JObject dataObj = JsonConvert.DeserializeObject<JObject>(jObject["data"].ToString());
                        if (dataObj != null)
                            resultBase.data = jObject["data"];
                        else
                            resultBase.data = null;
                    }
                    if (jObject["actionName"] != null)
                    {
                        resultBase.actionName = jObject["actionName"].ToString();
                    }
                }
                else
                {
                    resultBase.resultcode = -1;
                    resultBase.msg = jObject["msg"].ToString();
                    if (jObject["data"] != null)
                    {
                        JObject dataObj = JsonConvert.DeserializeObject<JObject>(jObject["data"].ToString());
                        if (string.IsNullOrEmpty(jObject["data"].ToString()) || dataObj == null)
                        {
                            resultBase.data = null;
                        }
                        else
                        {
                            resultBase.data = jObject["data"];
                        }
                    }

                    if (jObject["actionName"] != null)
                        resultBase.actionName = jObject["actionName"].ToString();
                }
            }
            catch (Exception ex)
            {
                resultBase.resultcode = -2;
                resultBase.msg = $"Http消息实时下发错误:{ex.ToString()},【请求URL：{url},返回Result：{result}】";
            }

            return resultBase;
        }
    }
}
