﻿using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace P2Pdemo.Models
{
    public class ResultHelper
    {
        public static ContentResult ToJson(Object obj)
        {
            string str;
            if (obj is String || obj is Char)
            {
                str = obj.ToString();
            }
            else
            {
                str = JsonConvert.SerializeObject(obj);
            }
            HttpHelper.HttpContext.Response.Headers.Add("Content-Length", str.Length.ToString());

            //Parking.Common.HttpContext.Current.Response.ContentLength = str.Length;

            var c = new ContentResult();
            c.Content = str;
            return c;

            //    Content("");
            //return str;
        }
        /// <summary>
        /// api返回结果
        /// </summary>
        /// <param name="deviceno">设备编号</param>
        /// <param name="resultcode">1-成功，其他为失败请求</param>
        /// <param name="msg">结果描述</param>
        /// <param name="data">结果集</param>
        /// <returns></returns>
        public static object SendResult(string deviceno, string resultcode, string msg, object data = null)
        {
            data = data ?? new { };
            string timestemp = DateTime.Now.ToString("yyyyMMddHHmmssfff");
            Dictionary<string, object> ret = new Dictionary<string, object>();
            ret.Add("resultcode", resultcode);
            ret.Add("msg", msg);
            ret.Add("deviceno", deviceno);
            ret.Add("timestemp", timestemp);
            ret.Add("data", data);

            string SignString = TyziTools.ApiSign.ToQuery(ret, false) + "key=" + deviceno;
            string sign = TyziTools.ApiSign.MD5Encrypt(SignString, System.Text.Encoding.UTF8);
            ret.Add("sign", sign);

            return ret;
        }

        /// <summary>
        /// api返回结果
        /// </summary>
        /// <param name="deviceno">设备编号</param>
        /// <param name="resultcode">1-成功，其他为失败请求</param>
        /// <param name="msg">结果描述</param>
        /// <param name="data">结果集</param>
        /// <returns></returns>
        public static ContentResult AESSendResult(string deviceno, string resultcode, string msg, object data = null)
        {
            string strData = data == null ? "" : TyziTools.ApiSign.ToString(data, true);

            string timestamp = DateTime.Now.ToString("yyyyMMddHHmmssfff");
            Dictionary<string, object> ret = new Dictionary<string, object>();
            ret.Add("resultcode", resultcode);
            ret.Add("msg", msg);
            ret.Add("deviceno", deviceno);
            ret.Add("timestamp", timestamp);
            ret.Add("data", strData);
            //签名
            string SignString = TyziTools.ApiSign.ToQuery(ret, false) + "key=" + deviceno;
            string sign = TyziTools.ApiSign.MD5Encrypt(SignString, System.Text.Encoding.UTF8);
            ret.Add("sign", sign);

            //数据体data加密
            if (!string.IsNullOrEmpty(strData))
                ret["data"] = System.Web.HttpUtility.UrlEncode(AESHelper.AesEncrypt(strData));

            string str = JsonConvert.SerializeObject(ret);
            HttpHelper.HttpContext.Response.Headers.Add("Content-Length", str.Length.ToString());
            var conresult = new ContentResult();
            conresult.Content = str;
            return conresult;
        }

        /// <summary>
        /// 校验签名
        /// </summary>
        /// <param name="param">请求参数数据，必须包含sign</param>
        /// <param name="bloder"></param>
        /// <param name="type">0-MD5加密方式，1-SHA256加密方式</param>
        /// <returns></returns>
        public static bool SignChecker(Dictionary<string, object> param, string signKey = "", int type = 0)
        {
            if (param == null) { return false; }

            string sign = param["sign"].ToString().ToUpper();
            string vSign = TyziTools.ApiSign.ToSign(param, param[signKey].ToString(), false, type).ToUpper();
            return sign == vSign;
        }



    }

    public class ResultOut
    {
        public string resultcode;
        public string msg;
        public string deviceno;
        public string timestemp;
        public string sign;
        public object data;
    }

    public class RRpcResult
    {
        public string id;
        public string code;
        public JObject data;
    }

    public class TcpResultOut
    {
        public string result;
        public string msg;
        public object data;
    }
}