﻿using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

namespace P2Pdemo.Models
{
    public class TyziTools
    {

        public class ApiSign
        {

            public static string ToString(object obj, bool isNull = false)
            {
                JsonSerializerSettings jsSet = new JsonSerializerSettings() { NullValueHandling = NullValueHandling.Ignore };
                if (isNull)
                    return JsonConvert.SerializeObject(obj);
                else
                    return JsonConvert.SerializeObject(obj, jsSet);
            }

            public static T ToObject<T>(string json)
            {
                try
                {
                    T model = JsonConvert.DeserializeObject<T>(json);
                    return model;
                }
                catch (Exception)
                {
                    return default(T);
                }
            }

            /// <summary>
            /// 检测时间与当前时间差是否在条件范围内(天)
            /// </summary>
            /// <param name="checkTime">被检测时间（yyyyMMddHHmmssfff）</param>
            /// <param name="dayParam">前后n天内</param>
            /// <returns></returns>
            public static bool CheckTimeRange(string checkTime, double dayParam = 1)
            {
                try
                {
                    IFormatProvider ifp = new System.Globalization.CultureInfo("zh-CN", true);
                    TimeSpan timeSpan = DateTime.Now.Subtract(DateTime.ParseExact(checkTime, "yyyyMMddHHmmssfff", ifp));
                    if (Math.Abs(timeSpan.TotalDays) > dayParam)
                    {
                        return true;
                    }
                    return false;
                }
                catch
                {
                    return false;
                }
            }

            /// <summary>
            /// MD5加密
            /// </summary>
            /// <param name="input">需要加密的字符串</param>
            /// <param name="encode">字符的编码</param>
            /// <returns></returns>
            public static string MD5Encrypt(string input, Encoding encode)
            {
                if (string.IsNullOrEmpty(input))
                {
                    return null;
                }
                MD5CryptoServiceProvider md5Hasher = new MD5CryptoServiceProvider();
                byte[] data = md5Hasher.ComputeHash(encode.GetBytes(input));
                StringBuilder sBuilder = new StringBuilder();
                for (int i = 0; i < data.Length; i++)
                {
                    sBuilder.Append(data[i].ToString("x2"));
                }
                return sBuilder.ToString();
            }

            /// <summary>
            /// SHA256函数
            /// </summary>
            /// /// <param name="str">原始字符串</param>
            /// <returns>SHA256结果</returns>
            public static string SHA256(string str, Encoding encode)
            {
                byte[] SHA256Data = encode.GetBytes(str);
                SHA256Managed Sha256 = new SHA256Managed();
                byte[] Result = Sha256.ComputeHash(SHA256Data);
                return Convert.ToBase64String(Result);  //返回长度为44字节的字符串
            }

            /// <summary>
            /// HMACSHA1算法加密并返回ToBase64String
            /// </summary>
            /// <param name="EncryptText">签名参数字符串</param>
            /// <param name="EncryptKey">密钥参数</param>
            /// <returns>返回一个签名值(即哈希值)</returns>
            public static string HMACSHA1Text(string EncryptText, string EncryptKey, Encoding encode)
            {
                //HMACSHA1加密
                HMACSHA1 hmacsha1 = new HMACSHA1();
                hmacsha1.Key = encode.GetBytes(EncryptKey);
                byte[] dataBuffer = encode.GetBytes(EncryptText);
                byte[] hashBytes = hmacsha1.ComputeHash(dataBuffer);
                return Convert.ToBase64String(hashBytes);
            }


            /// <summary>
            /// 将xml转化为Dictionary
            /// </summary>
            /// <param name="xml">标准键值xml</param>
            /// <param name="rootName">根节点名</param>
            public static Dictionary<string, object> ToDictionary(string xml, string rootName = "xml")
            {
                Dictionary<string, object> item = new Dictionary<string, object>();
                XmlDocument xd = new XmlDocument();
                xd.LoadXml(xml);
                XmlNode xroot = xd.SelectSingleNode(rootName);
                XmlNodeList xnList = xroot.ChildNodes;
                foreach (XmlNode xn in xnList)
                {
                    item.Add(xn.Name, xn.InnerText);
                }
                return item;
            }

            /// <summary>
            /// 将Dictionary转化为字典序的QueryString,字符串最后会多一个&   key=value&....
            /// </summary>
            /// <param name="item"></param>
            /// <param name="notNull">是否包含null值---true:不能包含null值，false:需要包含null</param>
            /// <returns></returns>
            public static string ToQuery(Dictionary<string, object> item, bool isNull)
            {
                string result = string.Empty;
                ArrayList akeys = new ArrayList(item.Keys);
                akeys.Sort(); //按字母顺序进行排序
                foreach (string skey in akeys)
                {
                    if (skey.ToLower().Equals("key") || skey.ToLower().Equals("sign")) continue;
                    if (isNull)
                    {
                        if (item[skey] != null || !string.IsNullOrEmpty(item[skey].ToString()))
                            result += string.Format("{0}={1}&", skey, item[skey]);
                    }
                    else
                    {
                        result += string.Format("{0}={1}&", skey, item[skey]);
                    }
                }
                return result;
            }

            /// <summary>
            /// 将xml转化为字典序的QueryString,字符串最后会多一个&    key=value&....
            /// </summary>
            /// <param name="xml"></param>
            /// <param name="notNull">是否包含null值---true:不能包含null值，false:需要包含null</param>
            /// <returns></returns>
            public static string ToQuery(string xml, bool isNull)
            {
                string result = string.Empty;
                Dictionary<string, object> item = ToDictionary(xml);
                result = ToQuery(item, isNull);
                return result;
            }

            /// <summary>
            /// json转换字典
            /// </summary>
            /// <param name="data"></param>
            /// <returns></returns>
            public static Dictionary<string, object> DicConvent(dynamic data)
            {
                try
                {
                    Dictionary<string, object> dic = JsonConvert.DeserializeObject<Dictionary<string, object>>(data.ToString());
                    return dic;
                }
                catch
                {
                    return null;
                }
            }

            /// <summary>
            /// 字典序的QueryString,字符串最后会多一个&
            /// </summary>
            /// <typeparam name="T"></typeparam>
            /// <param name="model"></param>
            /// <param name="toLower">属性名是否转为小写</param>
            /// <returns></returns>
            public static string ToQuery<T>(T model, bool toLower = false)
            {
                string result = string.Empty;
                Hashtable htb = new Hashtable();
                Type t = model.GetType();
                PropertyInfo[] PropertyList = t.GetProperties();
                foreach (PropertyInfo item in PropertyList)
                {
                    htb.Add(item.Name, item.GetValue(model, null));
                }
                ArrayList akeys = new ArrayList(htb.Keys);
                akeys.Sort(); //按字母顺序进行排序
                foreach (string skey in akeys)
                {
                    if (skey.ToLower().Equals("key") || skey.ToLower().Equals("sign")) continue;
                    var nskey = toLower ? skey.ToLower() : skey;
                    if (htb[skey] != null)
                        result += string.Format("{0}={1}&", nskey, htb[skey]);
                }
                return result;
            }

            public static string ToSign<T>(T model, string key, bool toLower = false, int type = 0)
            {
                string query = ToQuery<T>(model, toLower);
                return ToSign(query, key, type);
            }

            public static string ToSign(Dictionary<string, object> item, string key, bool isNull = false, int type = 0)
            {
                string query = ToQuery(item, isNull);
                return ToSign(query, key, type);
            }

            public static string ToSign(string xml, string key, bool isNull = false, int type = 0)
            {
                string query = ToQuery(xml, isNull);
                return ToSign(query, key, type);
            }

            /// <summary>
            /// 返回签名字符串
            /// </summary>
            /// <param name="query"></param>
            /// <param name="key">加密密钥</param>
            /// <param name="type">0:MD5,1:SHA256</param>
            /// <returns></returns>
            public static string ToSign(string query, string key, int type = 0)
            {
                if (!string.IsNullOrEmpty(key))
                    query = query + "key=" + key;
                if (type == 0)
                    return MD5Encrypt(query, Encoding.UTF8);
                else if (type == 1)
                    return SHA256(query, Encoding.UTF8);
                return "";
            }

        }
    }
}
